/*

DESCRIPTION:    Control firmware for development PV-panel based, interleave,
                flyback charger.

                Preliminary specifications:
                    + Input voltage: 25-40V DC
                    + Input current: maximum 20A
                    + Output voltage: 56V DC
                    + Output current: 9A
                    + Maximum continuous power: 500W
                    + Control mode: current
                    + Operating temperature: TBD

                An RS232 interface shared with ICSP provides an optional 
                console interface with a command set to monitor and control 
                operational parameters. Many parameters are stored in Non-
                Volatile Memory (NVM).

                RS232 settings: 115.2kbps, 8 data bits, 1 stop, no parity. The 
                console port is accessed over the micro-USB connector on the 
                PCB and is optimized for use with a GBIB-style control program.

                A status LED on the PCB indicates machine state:
                    + Red=fault
                    + Green=normal operation
                    + Red/Green=boot, HW or FW failure

SYNTAX:         n/a

KNOWN BUGS:     

COPYRIGHT:      (c) 2023, Brian Cornell, all rights reserved.

DEPENDENCIES:   see include file listing.

 TARGET MCU:    PIC16F1776

HW REVISION:    230209

VERSION:        1.0

MODIFICATION HISTORY:
 
    16 Mar 23, BWC, version 1 adapted from v1.0 branch of IF2B code.
    

*/


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * COMPILER & CLIB INCLUDES.
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <pic.h>
#include <xc.h>

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * FIRMWARE INCLUDES.
 * + binary:            boolean constants shared across firmware projects.
 * + sizes:             constants representing common object sizes, value 
 *                      limits, and characters.
 * + pic16f1776-map-v2: PIC specific peripheral alias, pin, NVM mapping, and
 *                      function declarations for shared functions.
 * + usart-v3:           usart function declarations & supporting constants,
 *                      objects, etc..
 * + config:            PIC config registers
 * + _xtal_freq:        sets the constant _XTAL_FREQ to the system oscillator
 *                      frequency (Fosc).  Used by the __delayxxx macros.
 * + usartc-v3:         EUSART peripheral user configuration constants.
 * + if2b:              main include file for this project.
 * 
 * PROJECT FOLDER ORGANIZATION & LOCATION OF INCLUDES.
 * /if2b-221017-fw-v1_0 this (IF2B) project.
 * /microchip/code      shared includes.
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#include <binary.h>                     //folder microchip/code
#include <sizes.h>                      //folder microchip/code
#include <pic16f1776-map-v2.h>          //folder microchip/code
#include <usart-v3.h>                   //folder microchip/code
#include "config.h"
#ifndef _XTAL_FREQ
#include "_xtal_freq.h"
#endif
#include "usartc-v3.h"
#include "pic16f1776-vmv-v2.h"
#include "pvc.h"


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * MODULE DATA.
 * + inv is the volts object storing input voltage acquisition data.
 * + outv is the volts object storing output voltage acquisition data.
 * + vdd is the volts object storing Vdd acquisition data.
 * + tp1 is the volts object storing the voltage analog of P1 temperature.
 * + tp2 is the volts object storing the voltage analog of P2 temperature.
 * + usart_nl is set TRUE to add a new line (NL) to the output of the 
 *   usart_wr_ui... functions.
 * + flag_ocp1 is set true by the ISR when an over-current is detected for P1.
 * + flag_ocp2 is set true by the ISR when an over-current is detected for P2.
 * + flag_cll is TRUE when closed loop is enabled.
 * + Flags cv1 &2 are set TRUE to indicate that valid console command 
 *   parameters have been set in function main objects cmdp1 / cmdp2.
 * + Flag flag_nvm is TRUE when the NVM update timer has expired.
 * + Flag flag_nvf is TRUE when NVM must be updated with a new fault code.
 * + sts holds the status code, if any, associated with the machine state.
 * + msc maintains the system timer clock ticks for one second.
 * + state is a bit-mapped byte indicating the machine state (refer to STATE 
 *   MANAGEMENT AND STATUS CODES in include file).
 * + tmr_nvm counts down the number of hours to the next NVM update.
 * + tmr_temp is a countdown timer for measuring converter temps. It is set to
 *   0 @ boot so temps are checked prior to starting the converter.
 * + occp1 & occp2 are the over-current counters maintained in the fault
 *   monitoring section of the RUN loop.
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
volts   inv={.channel=PIN_IA_INV, .mvr=S_RT_ADC_MV_H, .adc1v=S_QY_ADC_1V_H,\
            .scale=S_RT_VIN},
        outv={.channel=PIN_IA_OUTV, .mvr=S_RT_ADC_MV_H, .adc1v=S_QY_ADC_1V_H,\
            .scale=S_RT_VOUT},
        vdd={.channel=S_AL_ADCCH_VDD, .mvr=S_RT_ADC_MV_L,\
            .adc1v=S_QY_ADC_1V_L, .scale=S_RT_VDD},
        tp1={.channel=PIN_IA_TFP1, .mvr=S_RT_ADC_MV_H, .adc1v=S_QY_ADC_1V_H,\
            .scale=S_RT_TEMP},
        tp2={.channel=PIN_IA_TFP2, .mvr=S_RT_ADC_MV_H, .adc1v=S_QY_ADC_1V_H,\
            .scale=S_RT_TEMP};

extern __bit usart_nl;

__bit   flag_ocp1,
        flag_ocp2,
        flag_cll,
        flag_cv1,
        flag_cv2,
        flag_nvm,
        flag_nvf;
uint8_t sts=STS_NOF,
        msc=0,
        state,
        tmr_nvm=S_TM_NTMR,
        tmr_temp=0;
uint16_t occp1=0,
         occp2=0;


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * OPERATING CONFIGURATION MEMORY.
 * 
 * Non-volatile parameter storage & management. Object nvm_cfg resides in 
 * program flash at the top of the high-endurance section (to make the lower 
 * portion contiguous & available for code).  Object ram_cfg is the working 
 * copy in RAM.
 * 
 * Initialization of nvm_cfg & ram_cfg MUST match: these are system defaults and 
 * enable recovery after NVM corruption or to restore via console commands.  
 * Refer to detailed explanation of NVM management at beginning of function 
 * Initialize.
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
const nvmcfg nvm_cfg __at(0x1FE0)={.byte={S_DF_FLASH_0,S_DF_FLASH_1,\
    S_DF_FLASH_2,S_DF_FLASH_3,S_DF_FLASH_4,S_DF_FLASH_5,S_DF_FLASH_6,\
    S_DF_FLASH_7,S_DF_FLASH_8,S_DF_FLASH_9,S_DF_FLASH_10,S_DF_FLASH_11,\
    S_DF_FLASH_12,S_DF_FLASH_13,S_DF_FLASH_14,S_DF_FLASH_15,S_DF_FLASH_16,\
    S_DF_FLASH_17,S_DF_FLASH_18,S_DF_FLASH_19,S_DF_FLASH_20,S_DF_FLASH_21,\
    S_DF_FLASH_22,S_DF_FLASH_23,S_DF_FLASH_24,S_DF_FLASH_25,S_DF_FLASH_26,\
    S_DF_FLASH_27,S_DF_FLASH_28,S_DF_FLASH_29,S_DF_FLASH_30,S_DF_FLASH_31}};
nvmcfg ram_cfg={.byte={S_DF_FLASH_0,S_DF_FLASH_1,S_DF_FLASH_2,S_DF_FLASH_3,\
    S_DF_FLASH_4,S_DF_FLASH_5,S_DF_FLASH_6,S_DF_FLASH_7,S_DF_FLASH_8,\
    S_DF_FLASH_9,S_DF_FLASH_10,S_DF_FLASH_11,S_DF_FLASH_12,S_DF_FLASH_13,\
    S_DF_FLASH_14,S_DF_FLASH_15,S_DF_FLASH_16,S_DF_FLASH_17,S_DF_FLASH_18,\
    S_DF_FLASH_19,S_DF_FLASH_20,S_DF_FLASH_21,S_DF_FLASH_22,S_DF_FLASH_23,\
    S_DF_FLASH_24,S_DF_FLASH_25,S_DF_FLASH_26,S_DF_FLASH_27,S_DF_FLASH_28,\
    S_DF_FLASH_29,S_DF_FLASH_30,S_DF_FLASH_31}};



/*Function main.  Control program entry point (from root code) for control 
 * program. All aspects of operation begin & end here. At entry the 
 * Initialization function is called to configure peripherals, data objects, 
 * etc. The STATUS & PCON register values are saved in case an MCU trap was
 * encountered. 
 * 
 * The WDT (Watchdog) timer is enabled for hard fault recovery and the RUN loop 
 * entered: it is an infinite loop that consists of several discrete blocks that
 * manage all aspects of operation. They are ordered by dependencies (e.g. data
 * collection must happen before fault detection) and/or criticality (e.g. fault
 * shutdown before console service). Refer to the documentation within each 
 * block for details.
 * 
 * Returns:     never
 *
 * Parameters:  none
 *
*/
void main(void)
{
    /*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
     * FUNCTION DATA.
     * + cmdb points to the start of the console command buffer.
     * + counter is a general purpose loop counter and scratch variable.  It 
     *   WILL NOT be used for multiple iterations thru the RUN loop.
     * + reg_status is assigned the value of the STATUS register at boot.  Refer
     *   to the 'MCU FAILURE RECORDING' block for more detail.
     * + reg_pcon is assigned the value of the PCON register at boot.  Refer
     *   to the 'MCU FAILURE RECORDING' block for more detail.
     * + cmdb_sz is the size of the console command buffer.
     * + cmdp1 is the integer value of the first console command parameter.
     * + cmdp2 is the integer value of the second console command parameter.
     * + ttime is temporary 'safe' (not subject to corruption via updating in
     *   the ISR since it is larger than the MCU's native data size) storage of 
     *   the unit's total time.
     *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
    char *cmdb;
    uint8_t counter=0,
            reg_status,
            reg_pcon,
            cmdb_sz,
            cmdp1;
    uint16_t ttime,
             cmdp2,
             dcp1=0,
             dcp2=0;


    /*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
     * MCU FAILURE RECORDING.
     * 
     * If an MCU failure (e.g. stack overflow, watchdog, etc.) is detected at 
     * boot it's important to understand what caused the fault. The STATUS &
     * PCON registers are saved at the start of the main function. Once 
     * initialization is complete, but before the run loop, these are saved to 
     * object ram_cfg for writing to NVM if the MCU status code is set.
     * 
     * They will remain in NVM until another MCU trap occurs, the NVM status
     * code is cleared, or defaults restored. The console command for get NVM 
     * status will return the status code mnemonic along with the STATUS & PCON
     * registers. Register values are only recorded when an MCU status is set so
     * these will be 0x00 for any other code.
     +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
    reg_status=S_AL_REG_STATUS;
    reg_pcon=S_AL_REG_PCON;


    /*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
     * INITIALIZATION.
     * 
     * Function Initialize configures the MCU and peripherals.  Afterwards a 
     * delay provides settle time.  Local objects are initialized before
     * entering the RUN loop:
     * + Save STATUS & PCON registers in ram_cfg object for subsequent write to
     *   NVM if MCU error by the 'fault monitoring & recovery control' block
     *   within the RUN loop.
     +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
    Initialize();
    __delay_ms(S_TM_INT);

    if (sts == STS_MCU)
    {
        ram_cfg.byte[FLASH_AR_STATUS]=(unsigned char) reg_status;
        ram_cfg.byte[FLASH_AR_PCON]=(unsigned char) reg_pcon;
    }


    /*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
     * RUN LOOP PRIORITIZATION & WATCHDOG MANAGEMENT.
     * 
     * Once entered, the run-time loop's only exit points are a reboot or power
     * cycle. The loop consists of several discrete blocks responsible for 
     * data collection, monitoring, and control.  They are organized in order
     * of criticality and/or dependency of subsequent blocks.  Categorically the
     * order is:
     *      + Sensor calibration
     *      + I2C slave data queries
     *      + Data collection
     *      + Fault monitoring
     *      + Shut-down
     *      + NVM updating / fault recording
     *      + Maximum Power Point tracking
     *      + Fault recovery
     *      + Boot (autostart) and restart
     *      + Status LED
     *      + Console service
     * 
     * The watchdog timer is expected to be on in the run loop regardless of
     * state.  It should be set to a value that accounts for the longest loop
     * time with at least a 100% margin.  If it must be disabled in anticipation
     * of a long delay then it is the responsibility of that code block to 
     * re-enable.
     +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
    S_AL_WDT_ENABLE=TRUE;                   //Enable watchdog

    while (TRUE)                            //Begin RUN loop
    {
        CLRWDT();                           //Clear watchdog timer

#ifdef _OPT_LOOP_TIMING
        PIN_OD_STATUS^=TRUE;
#endif
        /*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
         * DATA COLLECTION.
         * 
         * This block measures system voltages & currents and COG (duty cycle) 
         * timing.
         * 
         * Duty cycle is captured using the timer gate pin timing function. The 
         * gate pin is connected to the COG-A output. To start timing the timer 
         * is cleared and the go bit set. Timing begins on the next rising edge 
         * of the COG-A pin and stops when it falls.
         * 
         * To eliminate fixed delays waiting for the timing measurement to
         * complete, measurement relies on the RUN loop timing exceeding it. 
         * Timing is saved to the dcxx objects prior to the start of the next 
         * measurement iteration. Hence, the reported DC is always one 'RUN
         * loop' behind.
         * 
         * Since temperature is a relatively slow moving parameter it is only
         * measured periodically to reduce average RUN loop execution time. 
         * tmr_temp is maintained by the system timer ISR.
         *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
        ReadVoltage(&inv);
        ReadVoltage(&outv);

        //Measure duty cycle
        S_AL_DCTEN_P1=FALSE;                    //Ensure timing has stopped
        S_AL_DCTEN_P2=FALSE;
        dcp1=S_AL_DCTMR_P1;                     //Save DC
        dcp2=S_AL_DCTMR_P2;
        S_AL_DCTMR_P1=0;                        //Clear (high byte is not used)
        S_AL_DCTMR_P2=0;
        S_AL_DCTEN_P1=TRUE;                     //Start capture
        S_AL_DCTEN_P2=TRUE;
        //__delay_us(S_TM_DC_RUN);                //Wait fixed time

        if (!(tmr_temp))                        //Time to check temps
        {
            ReadVoltage(&tp1);
            ReadVoltage(&tp2);
            tmr_temp=S_TM_TEMP;                 //Reset timer
        }


        /*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
         * FAULT MONITORING.
         * 
         * This block tests for various fault conditions that set the operating
         * STATE to recoverable (STE_RCVRF) or non-recoverable (STE_FAULT). With
         * recoverable faults the firmware will resume normal operation when the
         * recovery criteria are met. A non-recoverable fault requires a reboot
         * (via console) or power cycle.
         *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

        /*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
         * HIGH VOUT.
         *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
        if (outv.volt > S_LM_FLT_HOV)
        {
            flag_nvf=TRUE;
            state=STE_FAULT;
            sts=STS_HOV;
        }

        /*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
         * EXCESSIVE SWITCH OVER-CURRENT EVENTS.
         * 
         * A compile option is provided to set a non-recoverable fault
         * when the heuristic over-current count limit is exceeded. But,
         * in the intended application where autonomous operation is 
         * desired it should be disabled. Current mode control ensures
         * that the plant remains within its SOA, and the high-temp & 
         * low output voltage fault tests ensure safety and protect 
         * against damage (not to mention input/output fuses).
         * 
         * The counters remain active when fault detection is disabled
         * to allow console observation for trouble-shooting or 
         * adjustment of the limit.
         +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
        if (flag_ocp1)
        {
            flag_ocp1=FALSE;
            S_AL_OCIE_P1=TRUE;                  //Re-enable interrupt
#ifdef _OPT_OC_FAULT
            if (++occp1 > S_LM_FLT_OCC)         //Set fault if hysteresis limit exceeded
            {
                state=STE_FAULT;
                sts=STS_OCL;
                flag_nvf=TRUE;
                //break;
            }
#else
            ++occp1;
#endif
        }
        else if (occp1) --occp1;

        if (flag_ocp2)
        {
            flag_ocp2=FALSE;
            S_AL_OCIE_P1=TRUE;                  //Re-enable interrupt
#ifdef _OPT_OC_FAULT
            if (++occp2 > S_LM_FLT_OCC)         //Set fault if hysteresis limit exceeded
            {
                state=STE_FAULT;
                sts=STS_OCL;
                flag_nvf=TRUE;
                //break;
            }
#else
            ++occp2;
#endif
        }
        else if (occp2) --occp2;


        /*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
         * FAULT SHUT-DOWN.
         * 
         * Stop the converter in the event of a FAULT when in the RUN state. The
         * PRG GO bit identifies when the converter is running. When a fault is
         * detected object state is changed from RUN to the recoverable or
         * fault value.
         *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#ifdef _OPT_SD_FAULT
        if ((S_AL_PRGGO_P1) && !(state & STE_RUN))
        {
            ConverterOFF();
        }
#endif

        /*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
         * NVM STATUS RECORDING.
         * 
         * This block updates NVM with status codes when a fault occurs or to
         * update the unit's total time. Flag flag_nvm is set TRUE by the timing
         * block in the ISR when the NVM update timer, tmr_nvm, expires.
         * This is typically done 2x/day and balances keeping the total time 
         * current between power cycles with NVM lifetime.
         * 
         * Flag flag_nvf is set TRUE when a fault occurs that should be recorded
         * in NVM. Both the code and time are saved: to preserve NVM life, the
         * fault is only written to NVM once each hour. This is important for
         * faults that can persist.
         * 
         * Before writing NVM, Vdd is checked reasonably close to committing to
         * the NVM write, and if insufficient will keep trying until successful 
         * or the unit dies. Vdd hold-up time should exceed the execution time
         * of this block by at least a factor of two.
         +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
        if (flag_nvm | flag_nvf)
        {
            SAFEASSIGN(ttime, ram_cfg.word[FLASH_AR_TTIM]);
            //Clear fault flag if NVM write already done
            if ((flag_nvf) && (ram_cfg.byte[FLASH_AR_STS] == sts) &&\
                    (ram_cfg.word[FLASH_AR_TSTS] == ttime))
                flag_nvf=FALSE;

            ReadVoltage(&vdd);              //But always check because....

            //NVM write could be for fault or TT update.
            if ((vdd.volt > S_LM_VDD_NVM) && (flag_nvm | flag_nvf))
            {
                //Only update ram_cfg objects when committed to writing.
                ram_cfg.byte[FLASH_AR_STS]=sts;
                ram_cfg.word[FLASH_AR_TSTS]=ttime;
                writeFlashRow((uint16_t) &nvm_cfg, (unsigned char *) &ram_cfg);

                //Reset NVM update timer and clear NVM update flags.
                tmr_nvm=S_TM_NTMR;
                flag_nvm=FALSE;
                flag_nvf=FALSE;
            }
        }


        /*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
         * RECOVERABLE FAULT & IDLE RECOVERY MONITORING.
         * 
         * This block uses waterfall processing to clear FAULT & IDLE 
         * conditions. Recoverable fault tests are performed first. If all tests
         * pass then the fault condition is cleared: the machine state is set
         * to IDLE and the status code cleared.
         * 
         * IDLE recovery tests are then performed. Passing these tests, or 
         * skipping if the options are not set, sets the machine state to
         * RESTART for the START block to return the converter to its proper
         * operating state.
         *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#ifdef _OPT_RC_FAULT
        if (state & STE_RCRIE)
        {
            while (TRUE)
            {
                /*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                 *               *** FAULT RECOVERY TESTS ***
                 +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

                /*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                 *                *** IDLE RECOVERY TESTS ***
                 +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
                if (!(state & STE_IDLE))
                {
                    state=STE_IDLE;
                    sts=STS_NOF;
                }

                state=STE_RSTRT;
                break;
            }
        }
#endif


        /*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
         * BOOT AND RECOVERY START.
         * 
         * Starts the converter if auto-start is enabled. Otherwise sets operating
         * state to STOP.
         *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
        if (state & STE_BTRRT)
        {
            if (ram_cfg.byte[FLASH_AR_AUTO]) ConverterON();
            else state=STE_STOP;
        }


#ifndef _OPT_LOOP_TIMING
        /*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
         * LED MANAGEMENT.
         * Set the LED color based on machine state:
         *  + Red=[recoverable] fault
         *  + Green=stopped, idle, run, and technically boot but that state 
         *    cannot occur if the first RUN loop has made it this far.
         *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
        if (state & STE_RCRFT) PIN_OD_LED=TRUE;
        else PIN_OD_LED=FALSE;
#endif


        /*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
         * CONSOLE SERVICE.
         * 
         * IMPORTANT: do not place any code expected to execute on each RUN 
         * loop iteration beyond this point. The console service handler should 
         * always be placed at bottom of RUN loop.
         * 
         * This block services the user console accessed via the RS232 micro-USB
         * service port. All commands are two bytes in length with a single, 
         * optional, integer parameter. Refer to the definition of object 
         * CmdList for command strings, explanation, and values.
         * 
         * The command line accepts a 2-character command string followed by
         * up to two integer parameters. The cmdb object points to the NULL
         * terminated command string and the objects cmdp1 & cmdp2 contain the
         * integer parameters. Bit flags flag_cv1 & flag_cv2 are TRUE to 
         * indicate that the respective integer contains a value from the
         * command line.
         * 
         * The command structure is optimized to minimize processing time & code 
         * size balanced with not being too cryptic for the user. That said,
         * character echo is off and the commands provide no response on 
         * validity or success. No bounds checking is done with the exception of
         * the NVM byte address.
         * 
         * Care must be taken when entering NVM data to ensure the correct byte
         * is being updated. Further, 16-bit values are stored in little endian
         * format and the user must calculate the decimal equivalents of the
         * high/low hexadecimal values.
         * 
         * IMPORTANT: all set commands and NVM updates MUST be done with the 
         * converter stopped. NVM updates require an MCU reset to take effect.
         *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

        //Retrieve command buffer & size, skip processing if too small.
        if (usartStatRx() != USARTSTSRXRDY) continue;
        cmdb=(char *) usartGetRx();             //Get command buffer
        if ((cmdb_sz=(uint8_t) usartGetSz()) < CMDSIZSTR)   //Too small
        {
            usartSetRx(ENABLE);
            continue;
        }

        //Parse buffer: command string & optional integer parameters
        *(cmdb+CMDSIZSTR)=S_CH_NULL;            //Terminate command string
        for (counter=(CMDSIZSTR+1), flag_cv1=FALSE, flag_cv2=FALSE; \
                (counter < cmdb_sz); ++counter)
        {
            if (*(cmdb+counter) > S_CH_SPACE)
            {
                if (!(flag_cv1))                //Found first parameter
                {
                    flag_cv1=TRUE;
                    cmdp1=(uint8_t) atoi(cmdb+counter);
                }
                else if (!(flag_cv2))           //Found second parameter
                {
                    flag_cv2=TRUE;
                    cmdp2=(uint16_t) atoi(cmdb+counter);
                }
                for (; (*(cmdb+counter) > S_CH_SPACE); ++counter);
            }
        }

        /*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
         * PROCESS COMMANDS.
         * 
         * These don't take parameters.
         +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
        //Read firmware release
        if (!(strcmp(cmdb, CmdList[CMDREDREL])))
        {
            usart_nl=FALSE;
            usart_wr_ui16(VERMAJOR);
            usart_wr_chr(S_CH_DECIMAL);
            usart_wr_ui16(VERMINOR);
            usart_wr_chr(S_CH_DECIMAL);
            usart_nl=TRUE;
            usart_wr_ui16(VERBUILD);
        }
        //Restore NVM defaults
        else if (!(strcmp(cmdb, CmdList[CMDXNVDFL])))
        {
            writeFlashRow((uint16_t) &nvm_cfg, NULL);
        }
        //Single pulse
        else if (!(strcmp(cmdb, CmdList[CMDXSPULS])))
        {
            ConverterSP();
        }
        //Reset MCU
        else if (!(strcmp(cmdb, CmdList[CMDXRTMCU])))
        {
            /*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
             * Reset system; first do clean stop if running. Disable watchdog 
             * to prevent erroneous reboot from delay before reset, and clear
             * PCON bits to prevent error trap at boot.
             +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
            if (state & STE_RUN) ConverterOFF();
            S_AL_WDT_ENABLE=FALSE;
            S_AL_REG_PCON=S_LM_PCON_BT;
            __delay_ms(S_TM_RST);
            RESET();
        }
        /*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
         * All remaining commands require at least first parameter: test for 
         * this before parsing command.
         +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
        else if (flag_cv1)
        {
            //Get over-current count
            if (!(strcmp(cmdb, CmdList[CMDGETCNT])))
            {
                if (cmdp1) usart_wr_ui16(occp2);
                else usart_wr_ui16(occp1);
            }
            //Get [measured] duty cycle
            else if (!(strcmp(cmdb, CmdList[CMDGETDCY])))
            {
                if (cmdp1) usart_wr_ui16(dcp2);
                else usart_wr_ui16(dcp1);
            }
            //Get system status
            else if (!(strcmp(cmdb, CmdList[CMDGETSTS])))
            {
                if (cmdp1) usart_wr_ui16(sts);
                else usart_wr_ui16(state);
            }
            //Get temps
            else if (!(strcmp(cmdb, CmdList[CMDGETEMP])))
            {
                if (cmdp1) usart_wr_ui16(tp2.vmv);
                else usart_wr_ui16(tp1.vmv);
            }
            //Get volts
            else if (!(strcmp(cmdb, CmdList[CMDGETVOL])))
            {
                if (cmdp1) usart_wr_ui16(outv.vmv);
                else usart_wr_ui16(inv.vmv);
            }
            //Read NVM byte
            else if (!(strcmp(cmdb, CmdList[CMDREDBYT])))
            {
                if (cmdp1 < S_SZ_FLASHR)
                    usart_wr_ui16((uint16_t) ram_cfg.byte[cmdp1]);
            }
            //Read NVM word
            else if (!(strcmp(cmdb, CmdList[CMDREDWOR])))
            {
                if (cmdp1 < S_SZ_FLASHW)
                    usart_wr_ui16(ram_cfg.word[cmdp1]);
            }
            //Set NVM byte
            else if (!(strcmp(cmdb, CmdList[CMDSETBYT])))
            {
                if ((flag_cv2) && (cmdp1 < S_SZ_FLASHC) && \
                        (ram_cfg.byte[cmdp1] != (uint8_t) cmdp2))
                {
                    ram_cfg.byte[cmdp1]=(uint8_t) cmdp2;
                    writeFlashRow((uint16_t) &nvm_cfg,\
                            (unsigned char *) &ram_cfg);
                }
            }
            //Set converter on / off
            else if (!(strcmp(cmdb, CmdList[CMDSETCON])))
            {
                if (cmdp1) ConverterON();
                else
                {
                    ConverterOFF();
                    //Don't alter FAULT state if set!
                    if (!(state & STE_FAULT)) state=STE_STOP;
                }
            }
            //Set working duty cycle
            else if (!(strcmp(cmdb, CmdList[CMDSETWDC])))
            {
                if (cmdp1 <= S_DC_MAX)
                {
                    chgDutyCycle(0, (uint16_t) cmdp1);
                }
            }
            //Set NVM word
            else if (!(strcmp(cmdb, CmdList[CMDSETWOR])))
            {
                if ((flag_cv2) && (cmdp1 < (S_SZ_FLASHW-1)) && \
                        (ram_cfg.word[cmdp1] != cmdp2))
                {
                    ram_cfg.word[cmdp1]=cmdp2;
                    writeFlashRow((uint16_t) &nvm_cfg,\
                            (unsigned char *) &ram_cfg);
                }
            }
        }

        usartSetRx(ENABLE);                 //Look for new user input
    }
} //End function main


/* Function ConverterON. Starts converter; if already started it has no effect.
 * No pre-start diagnostic checks are done since this is expected to be
 * handled by the RUN loop and reflected in the machine state object.
 * 
 * Soft start is employed. The duty cycle is started at S_DC_MIN and incremented 
 * to the DC stored in NVM. If open loop this is the DC the unit will operate 
 * at; if closed the control loop will regulate DC up to the maximum set by 
 * NVM.
 * 
 * Returns:     none.
 *
 * Parameters:  none
 *
*/
void ConverterON()
{
    if (state & STE_BRS)
    {
        //Reset RUN loop control objects
        occp1=0;
        occp2=0;
        flag_ocp1=FALSE;
        flag_ocp2=FALSE;
        S_AL_OCIF_P1=FALSE;                     //Ensure P1 over-current clear
        S_AL_OCIF_P2=FALSE;                     //Ensure P2 over-current clear
        S_AL_OCIE_P1=TRUE;                      //Enable P1 over-current count
        S_AL_OCIE_P2=TRUE;                      //Enable P2 over-current count

        S_AL_DC_P1=S_DC_MIN;                    //Start @ min. DC
        S_AL_DC_P2=S_DC_MIN;                    //Start @ min. DC
        PWMLD=S_EN_PWMLD;                       //Load DC simultaneously
        while (PWMLD);                          //Wait for load to complete

        S_AL_PRGGO_P1=TRUE;                     //Start P1 slope comp
        S_AL_PRGGO_P2=TRUE;                     //Start P2 slope comp
        S_AL_COGAS_P1=FALSE;                    //Enable P1 COG
        S_AL_COGAS_P2=FALSE;                    //Enable P2 COG

        chgDutyCycle(S_DC_MIN, ram_cfg.byte[FLASH_AR_DC]);   //Ramp DC

        state=STE_RUN;                          //Set machine state to RUN
    }

    return;
} //End function ConverterON


/* Function ConverterSP. Diagnostic that enables converters for approximately
 * one switching period. A fixed (short) duty cycle is used to minimize the
 * chance for damage.
 * 
 * Nothing happens if the converter is not stopped. This feature may not work
 * properly if operating in closed loop. Machine state & status are not altered.
 * 
 * Returns:     none.
 *
 * Parameters:  none
 *
*/
void ConverterSP()
{
    if (state & STE_STOP)
    {
        //Clear DC so COGs can be enabled without consequence
        S_AL_DC_P1=S_DC_OFF;
        S_AL_DC_P2=S_DC_OFF;
        PWMLD=S_EN_PWMLD;
        while (PWMLD);                          //Wait for load to complete

        S_AL_COGAS_P1=FALSE;                    //Enable P1 COG
        S_AL_COGAS_P2=FALSE;                    //Enable P2 COG

        //Load single pulse DC
        S_AL_DC_P1=S_DC_SP;
        S_AL_DC_P2=S_DC_SP;
        PWMLD=S_EN_PWMLD;
        while (PWMLD);                          //Wait for load to complete

//        __delay_us(S_TM_DC_SP);                 //Wait one switching period
        __delay_us((S_TM_DC_SP)*100);

        //Stop immediately
        S_AL_COGAS_P1=TRUE;
        S_AL_COGAS_P2=TRUE;
    }

    return;
} //End function ConverterSP


/* Function ConverterOFF.  Stops converter:
 *  + All converter phases are simultaneously stopped by setting the duty cycle
 *    to zero.
 *  + The COGs & PRGs are then stopped.
 * 
 * Updating the machine state & status is the responsibility of the caller.
 * 
 * Returns:     none.
 *
 * Parameters:  none
 *
*/
void ConverterOFF()
{
    S_AL_DC_P1=S_DC_OFF;                        //Turn off with no DC
    S_AL_DC_P2=S_DC_OFF;                        //Turn off with no DC
    PWMLD=S_EN_PWMLD;                           //Load DC simultaneously
    while (PWMLD);                              //Wait for load to complete

    S_AL_COGAS_P1=TRUE;                         //Stop P1
    S_AL_COGAS_P2=TRUE;                         //Stop P2
    S_AL_PRGGO_P1=FALSE;                        //Stop P1 slope comp
    S_AL_PRGGO_P2=FALSE;                        //Stop P2 slope comp

    return;
} //End function ConverterOFF


/* Function chgDutyCycle. Moves the duty cycle from the start to final value 
 * incrementally for 'soft' transitions. The constant S_TM_DCDELAY sets the 
 * delay, in uS, between DC increments.
 * 
 * Returns:     none
 *
 * Parameters:  start=16-bit starting DC value. If zero (0) then P1's current 
 *              DC is used.
 *              final=16-bit final DC value
 *
*/
void chgDutyCycle(uint16_t start, uint16_t final)
{
    if (!(start)) start=S_AL_DC_P1;             //Start with current DC
    while (start != final)
    {
        if (start < final) ++start; else --start;
        S_AL_DC_P1=start;
        S_AL_DC_P2=start;
        PWMLD=S_EN_PWMLD;
        __delay_us(S_TM_DCDELAY);
    }

    return;
} //End function chgDutyCycle


/*Function InterruptExec.  MCU interrupt handler.  Manages all enabled
 * interrupts which should be ordered according to priority high-low:
 *  + Over-current event capture
 *  + System timer
 *  + I2C
 *  + USART
 *
 * Returns:     none
 *
 * Parameters:  none
 *
*/
void __interrupt() InterruptExec(void)
{
    static uint16_t sc=0;

    /*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
     * OVER-CURRENT EVENT CAPTURE. The negative transition of the over-current 
     * comparator generates an interrupt. This block captures the event, sets 
     * the respective bit object flag_oc TRUE, and disables further over-
     * current interrupts. The RUN loop is expected to monitor the flag and 
     * take appropriate action (including re-enabling the interrupt).
     * 
     * This approach avoids the problems associated with ISR-based over-current
     * schemes that attempt to capture each event: a sustained over-current 
     * generates an interrupt storm that results in the PIC resetting. Of 
     * course, this scheme expects that the hardware can endure sustained over-
     * current events without damage.
     *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
    if (S_AL_OCIE_P1 & S_AL_OCIF_P1)
    {
        flag_ocp1=TRUE;
        S_AL_OCIE_P1=FALSE;
        S_AL_OCIF_P1=FALSE;
    }
    if (S_AL_OCIE_P2 & S_AL_OCIF_P2)
    {
        flag_ocp2=TRUE;
        S_AL_OCIE_P2=FALSE;
        S_AL_OCIF_P2=FALSE;
    }

    /*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
     * SYSTEM TIMER. Non-precision timed background service. ~10 mS frequency.
     * Maintains various timing objects:
     *  + tmr_temp - timer to measure converter temps.
     *  + tmr_nvm - timer to update NVM total time.
     *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
    if (S_AL_SYST_IE & S_AL_SYST_IF)
    {
        if (++msc == S_QY_SYST_1S)                  //One second elapsed
        {
            msc=0;
            if (tmr_temp) --tmr_temp;               //Temp timer
            if (++sc == S_QY_SECHOUR)               //One hour elapsed
            {
                sc=0;
                ++ram_cfg.word[FLASH_AR_TTIM];      //Increment total time
                if (!(--tmr_nvm)) flag_nvm=TRUE;    //Time to update NVM
            }
        }

        S_AL_SYST_IF=FALSE;
    }

    /*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
     * USART HANDLER. Service RS232 console.
     *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
    usartISR();

    return;
} //End function InterruptExec


/*Function Initialize. Called immediately after boot by function main:
 *  + Sets system clock speed
 *  + Sets boot fault trap & tests
 *  + Validates NVM & loads to RAM
 *  + Configures pins, peripherals, interrupts
 *  + Starts system timer
 *
 * Returns:     none
 *
 * Parameters:  none
 *
*/
void Initialize(void)
{
    di();                               //No interrupts during initialization    

    //Initialize state flags and start with assumption of clean boot.
    /*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
     * INITIALIZE STATE FLAGS & STATUS CODE.
     * For code size efficiency a waterfall method is employed. The FAULT flag
     * is set and the status code cleared. This way, as initialization 
     * progresses, only the status code has to be set when a fault is 
     * encountered. At the end of this function, if the status code is clear,
     * the FAULT flag is cleared and the BOOT flag set.
     *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
    state=STE_FAULT;
    sts=STS_NOF;

    /*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
     * INITIALIZE SYSTEM CLOCK & WATCHDOG.
     *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
    OSCCON = CFG_OSC_C;                 //Configure system clock
    while (!(HFIOFL));                  //Wait for oscillator to stabilize
    WDTCON = CFG_WDT_CON;               //Watchdog


    /*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
     * BOOT & NVM PROTECTION AND MANAGEMENT.
     *
     * Recovering from CPU/firmware failures (e.g. watchdog, stack OF/UF, etc.)
     * and NVM corruption is critical for safety and field reliability.  While
     * neither should happen unless there's a true failure, statistically it's 
     * possible when operating in environmental extremes or the more likely
     * existence of a firmware bug.
     * 
     * The goal is to maintain safe operation, keep the unit intact, and record
     * the failure state (in that order).
     * 
     * DATA MANAGEMENT
     * A single row (32 bytes) of NVM data is used for the storage of tunable
     * parameters (e.g. current limit, output voltage, etc.). Object nvm_cfg
     * resides in flash (NVM) and is initialized with default values. During
     * normal operation object ram_cfg contains a copy of nvm_cfg and is used
     * to update nvm_cfg. It must also be initialized with the same default
     * values as nvm_cfg.
     * 
     * DATA VALIDATION
     * NVM data is validated with a simple 16-bit checksum stored in the last
     * two bytes of nvm_cfg.
     * 
     * BOOT PROCESS
     * The CPU boot status is always checked first to ensure that any abnormal
     * condition is identified before being cleared by subsequent execution.  
     * The PCON & STATUS registers are examined for reboot due to watchdog, 
     * stack OF/UF, and possibly other conditions.  If found the status object
     * sts is updated and it's expected that the main RUN loop halts normal 
     * operation.
     * 
     * The NVM checksum is checked even if a CPU fault condition has been
     * detected. It is read directly from NVM and compared to the NVM's stored
     * checksum.  If they match nvm_cfg is copied to ram_cfg. This ensures 
     * that when a CPU fault is recorded it will not overwrite NVM with default
     * parameters.
     * 
     * If there's a mismatch ram_cfg retains it's default values and the status
     * code will be updated if not already set by the CPU checks.
     * 
     * The remainder of the initialization function continues unaltered and
     * when complete the main function enters the RUN loop. The converter
     * should not start if any status code other than BOOT is set. A code block 
     * in the main function loop is responsible for storing error codes, 
     * resulting from the boot process, or otherwise, in NVM.
     * 
     * Since the MCU's architecture requires the entire NVM row to be written
     * it's critical that ram_cfg contains valid data. This is why it's not
     * updated with nvm_cfg when checksum's don't match. By writing the status
     * code it also ensures that if NVM was truly corrupt it will now contain
     * valid data (provided there's not an MCU flash failure) and there's a 
     * chance of the unit returning to operation when power cycled.
     *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
    if (((S_AL_REG_PCON & S_LM_PCON_EX) != S_LM_PCON_BT) || \
            ((S_AL_REG_STATUS & S_LM_STAT_EX) != S_LM_STAT_BT))
    {
        sts=STS_MCU;
    }

    if (nvmChksum((unsigned char *) &nvm_cfg) != (unsigned int) \
            nvm_cfg.word[S_AR_FLCS_W])
    {
        if (sts == STS_NOF) sts=STS_NVM;
    }
    else memcpy((void *) &ram_cfg, (void *) &nvm_cfg, S_SZ_FLASHR);


    /*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
     * INITIALIZE CONTROL DATA:
     * + flag_cll is set to the stored NVM value for closed loop operation.
     * + usart_nl is set TRUE to add new line character to console output.
     *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
    flag_cll=(__bit) ram_cfg.byte[FLASH_AR_LOOP];
    usart_nl=TRUE;


    /*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
     * SET PORT CONFIGURATION:
     * + Set digital latch initial states
     * + Pin in/out-put states
     * + Analog / digital
     * + Open drain (tri-state)
     * + Slew rate
     * + Logic input levels
     * 
     * Important: weak pull-ups are not used (OPTION_REG WPUEN is left @ default
     * of disabled) to avoid power-up spurious switching.
     *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
    PORTA=S_IN_PORTA;
    PORTB=S_IN_PORTB;
    PORTC=S_IN_PORTC;
    //Port A
    TRISA=IO_PORTA;
    ANSELA=AD_PORTA;
    ODCONA=OD_PORTA;
    SLRCONA=SR_PORTA;
    INLVLA=IL_PORTA;
    //Port B
    TRISB=IO_PORTB;
    ANSELB=AD_PORTB;
    ODCONB=OD_PORTB;
    SLRCONB=SR_PORTB;
    INLVLB=IL_PORTB;
    //Port C
    TRISC=IO_PORTC;
    ANSELC=AD_PORTC;
    ODCONC=OD_PORTC;
    SLRCONC=SR_PORTC;
    INLVLC=IL_PORTC;


    /*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
     * Initialize USART for command console.
     * 
     * This should be done at end of port / pin config since it configures PPS
     * for the USART. Console echo OFF.
     *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
    usartSetRx(ENABLE);
    usartSetEcho(FALSE);


    /*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
     * CONFIGURE FVR & DACs:
     * 1=Vref for Vout control loop
     * 2=Vref for current limit
     * 5=Vref for MPP control loop
     * 3=Vdd
     *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
    FVRCON=CFG_FVR_C;

    DAC1CON0=CFG_DAC_1_C;
    DAC1REFH=ram_cfg.byte[FLASH_AR_VSET_H];
    DAC1REFL=ram_cfg.byte[FLASH_AR_VSET_L];
    DAC1LD=TRUE;                                //Load reference value

    DAC2CON0=CFG_DAC_2_C;
    DAC2REFH=ram_cfg.byte[FLASH_AR_CLIM_H];
    DAC2REFL=ram_cfg.byte[FLASH_AR_CLIM_L];
    DAC2LD=TRUE;                                //Load reference value

    DAC5CON0=CFG_DAC_5_C;
    DAC5REFH=ram_cfg.byte[FLASH_AR_VMPP_H];
    DAC5REFL=ram_cfg.byte[FLASH_AR_VMPP_L];
    DAC5LD=TRUE;                                //Load reference value

    DAC3CON0=CFG_DAC_3_C;
    DAC3REF=CFG_DAC_3_R;


    /*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
     * CONFIGURE ADC:  enable and connect to first measurement source in RUN 
     * loop. It's expected that all further management of ADC is done thru 
     * function ReadVoltage.
     *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
    ADCON0=CFG_ADC_CON_0;
    ADCON1=CFG_ADC_CON_1;
    ADCON2=CFG_ADC_CON_2;


    /*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
     * CONFIGURE OP-AMPS:
     * 2=Vout error amp
     * 3=MPP error amp
     *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
    OPA2ORS=CFG_OPA_2_O;
    OPA2NCHS=CFG_OPA_2_N;
    OPA2PCHS=CFG_OPA_2_P;
    OPA2CON=CFG_OPA_2_C;

    OPA3ORS=CFG_OPA_3_O;
    OPA3NCHS=CFG_OPA_3_N;
    OPA3PCHS=CFG_OPA_3_P;
    OPA3CON=CFG_OPA_3_C;


    /*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
     * CONFIGURE COMPARATORS:
     * 1=P2 slope compensation
     * 2=P2 current limit
     * 3=P1 slope compensation
     * 4=P1 current limit
     *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
    CM1CON1=CFG_CM_1_C1;
    CM1NSEL=CFG_CM_1_N;
    CM1PSEL=CFG_CM_1_P;
    CM1CON0=CFG_CM_1_C0;

    CM2CON1=CFG_CM_2_C1;
    CM2NSEL=CFG_CM_2_N;
    CM2PSEL=CFG_CM_2_P;
    CM2CON0=CFG_CM_2_C0;

    CM3CON1=CFG_CM_3_C1;
    CM3NSEL=CFG_CM_3_N;
    CM3PSEL=CFG_CM_3_P;
    CM3CON0=CFG_CM_3_C0;

    CM4CON1=CFG_CM_4_C1;
    CM4NSEL=CFG_CM_4_N;
    CM4PSEL=CFG_CM_4_P;
    CM4CON0=CFG_CM_4_C0;


    /*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
     * CONFIGURE TIMERS:
     * 3=P1 DC measurement
     * 5=P2 DC measurement
     * SYST=system chrono timing
     *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
    T3CON = CFG_TMR_3_CON;
    T3GCON = CFG_TMR_3_GCON;

    T5CON = CFG_TMR_5_CON;
    T5GCON = CFG_TMR_5_GCON;

    S_AL_SYST_CLKCON=CFG_SYST_CLK;
    S_AL_SYST_HLT=CFG_SYST_HLT;
    S_AL_SYST_RST=CFG_SYST_RST;
    S_AL_SYST_PR=CFG_SYST_PR;
    S_AL_SYST_CON=CFG_SYST_CON;


    /*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
     * CONFIGURE PWMs:
     * 5=P1
     * 6=P2
     *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
    PWM5INTE=CFG_PWM_5_INTE;
    PWM5CLKCON=CFG_PWM_5_CLK;
    PWM5LDCON=CFG_PWM_5_LDCON;
    PWM5OFCON=CFG_PWM_5_OFCON;
    PWM5PH=CFG_PWM_5_PHASE;
    PWM5DC=(uint16_t) ram_cfg.byte[FLASH_AR_DC];
    PWM5PR=CFG_PWM_5_PERIOD;
    PWM5OF=CFG_PWM_5_OFFSET;
    PWM5TMR=CFG_PWM_5_TIMER;
    PWM5CON=CFG_PWM_5_CON;

    PWM6INTE=CFG_PWM_6_INTE;
    PWM6CLKCON=CFG_PWM_6_CLK;
    PWM6LDCON=CFG_PWM_6_LDCON;
    PWM6OFCON=CFG_PWM_6_OFCON;
    PWM6PH=CFG_PWM_6_PHASE;
    PWM6DC=(uint16_t) ram_cfg.byte[FLASH_AR_DC];
    PWM6PR=CFG_PWM_6_PERIOD;
    PWM6OF=CFG_PWM_6_OFFSET;
    PWM6TMR=CFG_PWM_6_TIMER;
    PWM6CON=CFG_PWM_6_CON;


    /*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
     * CONFIGURE PRGS:
     * 1=P1 slope comp
     * 2=P2 slope comp
     *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
    PRG1FTSS=CFG_PRG_1_FTSS;
    PRG1RTSS=CFG_PRG_1_RTSS;
    PRG1CON2=ram_cfg.byte[FLASH_AR_SCOMP];
    PRG1INS=CFG_PRG_1_INS;
    PRG1CON1=CFG_PRG_1_CON1;
    PRG1CON0=CFG_PRG_1_CON0;

    PRG2FTSS=CFG_PRG_2_FTSS;
    PRG2RTSS=CFG_PRG_2_RTSS;
    PRG2CON2=ram_cfg.byte[FLASH_AR_SCOMP];
    PRG2INS=CFG_PRG_2_INS;
    PRG2CON1=CFG_PRG_2_CON1;
    PRG2CON0=CFG_PRG_2_CON0;


    /*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
     * CONFIGURE COGs:
     * 1=P1
     * 2=P2
     * 
     * NVM dictates loop state. CON0 registers are loaded last to enable with 
     * set configuration.
     *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
    COG1DBR=CFG_COG_1_DBR;
    COG1DBF=CFG_COG_1_DBF;
    COG1BLKR=CFG_COG_1_BLKR;
    COG1BLKF=CFG_COG_1_BLKF;
    COG1PHR=CFG_COG_1_PHR;
    COG1PHF=CFG_COG_1_PHF;
    COG1ASD1=CFG_COG_1_ASD1;
    COG1ASD0=CFG_COG_1_ASD0;
    COG1RIS0=CFG_COG_1_RIS0;
    COG1RIS1=CFG_COG_1_RIS1;
    COG1FIS1=CFG_COG_1_FIS1;
    COG1RSIM0=CFG_COG_1_RSIM0;
    COG1RSIM1=CFG_COG_1_RSIM1;
    COG1FSIM0=CFG_COG_1_FSIM0;
    COG1FSIM1=CFG_COG_1_FSIM1;
    COG1CON1=CFG_COG_1_CON1;

    COG2DBR=CFG_COG_2_DBR;
    COG2DBF=CFG_COG_2_DBF;
    COG2BLKR=CFG_COG_2_BLKR;
    COG2BLKF=CFG_COG_2_BLKF;
    COG2PHR=CFG_COG_2_PHR;
    COG2PHF=CFG_COG_2_PHF;
    COG2ASD1=CFG_COG_2_ASD1;
    COG2ASD0=CFG_COG_2_ASD0;
    COG2RIS0=CFG_COG_2_RIS0;
    COG2RIS1=CFG_COG_2_RIS1;
    COG2FIS1=CFG_COG_2_FIS1;
    COG2RSIM0=CFG_COG_2_RSIM0;
    COG2RSIM1=CFG_COG_2_RSIM1;
    COG2FSIM0=CFG_COG_2_FSIM0;
    COG2FSIM1=CFG_COG_2_FSIM1;
    COG2CON1=CFG_COG_2_CON1;

    if (ram_cfg.byte[FLASH_AR_LOOP])
    {
        SETLOOPCLOSED();
    }
    else
    {
        SETLOOPOPEN();
    }
    COG1CON0=CFG_COG_1_CON0;
    COG2CON0=CFG_COG_2_CON0;


    /*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
     * CONFIGURE CLCS:
     *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/


    /*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
     * MAP PERIPHERAL PINS:
     * + PWMs
     * + PRG triggers
     * + Gate drives
     * + Duty cycle measurement
     *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
    PPS_P_P5P1=PPS_M_P5P1;                      //P1 PWM
    PRG1RPPS=PPS_I_P1P1;                        //Ramp rising trigger
    PRG1FPPS=PPS_I_P1P1;                        //Ramp falling trigger

    PPS_P_P6P2=PPS_M_P6P2;                      //P2 PWM
    PRG2RPPS=PPS_I_P2P2;                        //Ramp rising trigger
    PRG2FPPS=PPS_I_P2P2;                        //Ramp falling trigger

#ifdef _OPT_ENA_P1
    PPS_P_C1P1=PPS_M_C1P1;                      //P1 gate drive
#endif
    T3GPPS=PPS_I_T3P1;                          //DC measurement

#ifdef _OPT_ENA_P2
    PPS_P_CGP2=PPS_M_CGP2;                      //P2 gate drive
#endif
    T5GPPS=PPS_I_T5P2;                          //DC measurement



    /*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
     * INITIALIZATION COMPLETE:
     * + Test for fault and if none set the STATE flags accordingly
     * + Enable peripheral & global interrupts
     * + Enable peripheral interrupts serviced by ISR
     * + Start system timer
     *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
    if (sts == STS_NOF) state=STE_BOOT;
    else flag_nvf=TRUE;

    PEIE=ENABLE;                        //Peripheral interrupts
    ei();                               //Global interrupts
    S_AL_SYST_IE=TRUE;                  //Enable system timer interrupt
    S_AL_SYST_EN=TRUE;                  //Enable system timer

    return;
} //End function Initialize



//End of file